#!/bin/sh -x
SOFT_VERSION="V1.0.0 2022/07/12"
SHELL_DIR=$(dirname $(readlink -f "$0"))	#shell脚本当前目录

######################## 菜单 ################################
Usage()
{
	cat << USAGE
Usage :
  sh GeneralCheck.sh -c    ---- check all iae
USAGE
	exit
}

####################### 退出 #################################
My_Exit()
{
    echo "the program will exit"
	# $1 : exit code
	# /bin/rm -rf ${PID_FILE} 2>/dev/null
	# cd ${OLD_PATH}
	exit $1
}

################### 安装/卸载sshpass ##############################
Install_Sshpass()
{
	if [ $1 ]
	then
		retry_count=$1
	else
		retry_count=0
	fi
	if [ ${retry_count} -eq 2 ]
	then
		#两次安装失败，退出脚本
		echo "sshpass install failed" >> ${Check_Log}
		Input_Error "install sshpass failed" && My_Exit 1
	fi
	# 已安装sshpass，不在安装 #
	cmd=` sshpass -V 2>/dev/null `
	if [ $? -ne 0 ]
	then
		Input_Info "installing sshpass, please wait"
		OLD_PATH=` pwd `
		Cmd "cd ${SHELL_DIR}/sshpass*"
		Cmd "./configure"
		Cmd "make"
		Cmd "make install"
		if [ $? -ne 0 ]
		then
			#第一次安装失败更新配置，尝试第二次安装
			cmd=` autoreconf 2>&1`
			((retry_count++))
			[ ${retry_count} -eq 1 ] && Output_Warning "fist install sshpass failed, will install for next"
			Install_Sshpass ${retry_count}
		fi
		cd ${OLD_PATH}
	fi
}
Uninstall_Sshpass()
{
	echo "will uninstall sshpass"
}
#################### 格式化输出 ******************************
Output_Error()
{
	echo -e "[error] $1"
}
Output_Warning()
{
	echo -e "\033[33m\033[1m[warning]\033[0m $1"
}
Output_Info()
{
	echo -e "\033[32m\033[1m[INFO]\033[0m $1"
}
Output_Test()
{
    echo -e "test: $1"
}
#本地命令#
Cmd(){
	$1 2>&1
	if [ $? -ne 0 ]
	then
		return 1
	fi
}
# 远程命令 #
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码
##$5 远程执行的命令
Ssh_Cmd(){
	sshpass -p $4 ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -p $2 $3@$1 "$5"
}
#################### 系统相关信息收集 ########################
###################### IAM 配置检查 ##########################
Check_Iam()
{	
	Output_Info "checking iam"
    Iam_Ver_Info=` iamcfgtool.sh -v `
	Iam_Ver_Info_Arr=(${Iam_Ver_Info// / })
	Iam_Hcverm=${Iam_Ver_Info_Arr[3]}
	cmd=` mv ${Check_Result_File} ${Check_Result_File}.bak 2>/dev/null `
	cmd=` mv ${Check_Result_File} ${Check_Result_File}.bak 2>/dev/null `
	#echo 	"Please Input IAM IP"
	#read Iam_Ip
	
	## 配置IAM免密登录
	# echo "#######BEGIN ADDED FOR IA CHECK########" >> ${Iam_Sshd_Config}
	# Sshd_Config_Items=("PubkeyAuthentication" "RSAAuthentication" "AuthorizedKeysFile" "PasswordAuthentication")
	# for item in ${!Sshd_Config_Items[@]}
	# do
		# Pub_Key_Auth_Rst=` grep -rn "[ ]*${Sshd_Config_Items[${item}]}[ ]*" ${Iam_Sshd_Config} `	# 配置文件中是否有当前配置项
		# while read line
		# do
			# num=${line%%:*}
			# config=${line##*:}
			# cmd=` sed -i "${num}c #${config}" ${Iam_Sshd_Config}` 	# 给原配置添加注释
		# done <<< "${Pub_Key_Auth_Rst}"
	# done
	# echo "PubkeyAuthentication yes" >> ${Iam_Sshd_Config} 
	# echo "RSAAuthentication yes" >> ${Iam_Sshd_Config} 
	# echo "AuthorizedKeysFile ${Iam_Authorized_keys}" >> ${Iam_Sshd_Config} 
	# echo "PasswordAuthentication yes" >> ${Iam_Sshd_Config} 
	# echo "#######END ADDED FOR IA CHECK########" >> ${Iam_Sshd_Config}
}
###################### IA 配置检查 ##########################
###########芯片资源收集#########
##参数列表：
##$1 板卡ip
##$2 板卡ssh用户名
##$3 板卡ssh密码
##$4 板卡ssh端口
##$5 昆仑2芯片号
Check_Kunlun2_Chip_Info()
{
	Output_Info "Begin Check Chip $1-$5"
	echo "Begin Check Chip $1-$5" >> ${Check_Result_File}
	echo "Begin Check Chip $1-$5" >> ${Check_Log}
	# Ssh_Cmd $1 $2 $3 $4 " ssh ${Board_Chip_IP[$1-$5]} \"ls\" "
	# 昆仑2磁盘使用情况
	echo >> ${Check_Log}
	echo "$1-$5 disk use info" >> ${Check_Log}
	disk_info=` Ssh_Cmd $1 $2 $3 $4 " ssh ${Board_Chip_IP[$1-$5]} \"df -h\" " 2>/dev/null `
	echo "${disk_info}" >> ${Check_Log}
	Check_Disk "${disk_info}"
	echo >> ${Check_Log}
	
	#昆仑2内存使用情况
	echo >> ${Check_Log}
	echo "	$1-$5 memmory use info" >> ${Check_Log}
	free_info=` Ssh_Cmd $1 $2 $3 $4 " ssh ${Board_Chip_IP[$1-$5]} \"free\" " 2>/dev/null `
	echo "${free_info}" >> ${Check_Log}
	free_info=`grep Mem <<< ${free_info}`
	Check_Free "${free_info}"
	echo >> ${Check_Log}
	
	# 输出昆仑2进程文件信息
	all_file_info=` Ssh_Cmd $1 $2 $3 $4 " ssh ${Board_Chip_IP[$1-$5]} \"ls -alh /opt/wise/ia12k/bin | grep ^-\" " 2>/dev/null `  
	iae_tp_info=` Ssh_Cmd $1 $2 $3 $4 " ssh ${Board_Chip_IP[$1-$5]} \"ls -alh /opt/wise/ia12k/bin | grep iae_tp\" " 2>/dev/null `
	iae_tp_info_arr=("${iae_tp_info}")
	iae_tp_time="${iae_tp_info_arr[5]}-${iae_tp_info_arr[6]}-${iae_tp_info_arr[7]}"
	check_process_file_k2="true"
	while read line
	do
		file_item_arr=(${line})
		file_name="${file_item_arr[8]}"
		file_time="${file_item_arr[5]}-${file_item_arr[6]}-${file_item_arr[7]}"
		md5=` Ssh_Cmd $1 $2 $3 $4 " ssh ${Board_Chip_IP[$1-$5]} \"md5sum /opt/wise/ia12k/bin/${file_name}\" " `
		md5_info_arr=(${md5})
		file_info="${line}  ${md5_info_arr[0]}"
		echo "${file_info}" >> ${Check_Log}
		if [ "${iae_tp_time}" != "${file_time}" ]
		then
			Output_Warning "$1-$5 have a process(/opt/wise/ia12k/process/bin/${file_name}) for backup. \nmaybe have a version or patch not be installed"
			echo "$1-$5 have a process(/opt/wise/ia12k/process/bin/${file_name}) for backup." >> ${Check_Result_File}
			echo "$1 maybe have a version or patch not be installed" >> ${Check_Result_File}
			check_process_file_k2="false";
			# "存在替换进程，可能有未实施的补丁" >> ${Check_Result_File}
			# "${line}" >> ${Check_Result_File} 
		fi
	done <<< "${all_file_info}"
	echo >> ${Check_Log}
	if [ "${check_process_file_k2}" == "true" ]
	then
		Output_Info "$1-$5 check process file is [ok]"
		echo "$1-$5 check process file is [ok]" >> ${Check_Result_File}
	fi
	
	# 检查core是否开启
	iae_pid=` cat /opt/wise/ia12k/run/iae.pid `
	limits=` cat /proc/${iae_pid}/limits `
	core_size_info=` cat /proc/${iae_pid}/limits | grep "core file size" `
	core_size_info_arr=("${core_size_info// / }")
	soft_limit=${core_size_info_arr[2]}
	has_set_core="true"
	have_core="false"
	echo "${limits}" >> ${Check_Log} 
	if [ ${soft_limit} != "unlimited" ]
	then
		has_set_core="false"
		echo "$1-$5 maybe not open core" >> ${Check_Result_File}
		Output_Warning "$1-$5 maybe not open core"
	else
		# core开启检查有没有core产生
		core_file_all=` ll /opt/wise_iae/ia12k/data*/corefiles/ | grep ^- `
		if [ $? eq 0 ]
		then
			have_core="true"
			echo "$1-$5 process maybe have a exception" >> ${Check_Result_File}
			echo "${core_file_all}" >> ${Check_Result_File}
			Output_Error "$1-$5 process maybe have a exception"
		fi
	fi
	if [ "${has_set_core}" != "true" ]
	then
		echo "$1-$5 has set core [ok]" >> ${Check_Result_File}
		Output_Info "$1-$5 process maybe have a exception"
	else
		if [ "${have_core}" == "false" ]
		then
			echo "$1-$5 has not core file [ok]" >> ${Check_Result_File}
			Output_Info "$1-$5 has not core file [ok]"
		fi
	fi
	##配置昆仑2芯片免密登录IAM
	##昆仑2芯片生成公私秘钥对
	# echo ${Board_Chip_IP[$1-$5]}
	# Ssh_Cmd $1 $2 $3 $4 " /bin/expect << EOF
							# spawn ssh ${Board_Chip_IP[$1-$5]} \"ssh-keygen\"
							# expect {
								# \"id_rsa):\"
									# {
										# send \"/root/.ssh/iaCheck\n\"
										# expect {
											# \"Overwrite (y/n)?\"
											# {
												# send \"y\n\"
												# expect \"passphrase)\" {
													# send \"\n\"
													# expect \"again:\" {send \"\n\"}
												# }
											# }
											# \"passphrase)\" 
											# {
												# send \"\n\"
												# expect \"again:\" {send \"\n\"}
											# }
										# }
									# }
								# }
							# expect \"EOF\""
	##昆仑2芯片公钥添加至IAM的授权登录文件
	# Output_Info "${Board_Chip_IP[$1-$5]} 免密登录"
	# Ssh_Cmd $1 $2 $3 $4 "ssh ${Board_Chip_IP[$1-$5]} \"cat /root/.ssh/iaCheck.pub\"" >> "${Iam_Authorized_keys}"
	
	##收集日志信息
	# mkdir ${SHELL_DIR}/CheckResult/$1-$5/
	# Ssh_Cmd $1 $2 $3 $4 " ssh ${Board_Chip_IP[$1-$5]} \"scp -o StrictHostKeyChecking=no -i ${Kunlun2_Key_File} /var/log/syslog root@${Iam_Ip}:/${SHELL_DIR}/CheckResult/$1-$5/\" "
	# Ssh_Cmd $1 $2 $3 $4 " ssh ${Board_Chip_IP[$1-$5]} \"scp -o StrictHostKeyChecking=no -i ${Kunlun2_Key_File} /var/log/ialog/iae00.log root@${Iam_Ip}:/${SHELL_DIR}/CheckResult/$1-$5/\" "
	# all_file_info=Ssh_Cmd $1 $2 $3 $4 " ssh ${Board_Chip_IP[$1-$5]} \"ls -al |grep ^-\" "
}
Check_Chip_Hisi_Info(){
	echo >> ${Check_Log}
	Output_Info "$1 - $5 Begin Check Hisi Chip "
	echo "Begin Check Chip $1-$5" >> ${Check_Result_File}
	echo "$1 - $5 Begin Check Chip " >> ${Check_Log}
	# Ssh_Cmd $1 $2 $3 $4 " ssh ${Board_Chip_IP[$1-$5]} \"ls\" "
	# 昆仑3磁盘使用情况
	echo "$1 - $5 芯片磁盘使用情况" >> ${Check_Log}
	disk_info=` Ssh_Cmd $1 $2 $3 $4 " ssh ${Board_Chip_IP[$1-$5]} \"df -h\" " 2>/dev/null `
	echo "${disk_info}" >> ${Check_Log}
	Check_Disk "${disk_info}"
	
	# 昆仑3内存使用情况
	echo >> ${Check_Log}
	echo "$1 - $5 芯片内存使用情况" >> ${Check_Log}
	free_info=` Ssh_Cmd $1 $2 $3 $4 " ssh ${Board_Chip_IP[$1-$5]} \"free\" " 2>/dev/null `
	echo "${free_info}" >> ${Check_Log}
	free_info=` grep Mem <<< "${free_info}" `
	Check_Free "${free_info}"
	
	# 输出昆仑3进程文件信息
	# echo "Board_Chip_IP=${Board_Chip_IP[$1-$5]}"
	# all_file_info=` Ssh_Cmd $1 $2 $3 $4 \"ls -alh /opt/wise_iae/ia12k/process/bin/ | grep ^-\" " 2>/dev/null `  
	# iae_tp_info=` Ssh_Cmd $1 $2 $3 $4 \"ls -alh /opt/wise_iae/ia12k/process/bin/ | grep iae_tp\" " 2>/dev/null `
	# iae_tp_info_arr=("${iae_tp_info}")
	# iae_tp_time="${iae_tp_info_arr[5]}-${iae_tp_info_arr[6]}-${iae_tp_info_arr[7]}"
	# echo "iae_tp_time=${iae_tp_time}"
	# echo >> ${Check_Result_File}
	# while read line
	# do
		# file_item_arr=(${line})
		# file_name="${file_item_arr[8]}"
		# file_time="${file_item_arr[5]}-${file_item_arr[6]}-${file_item_arr[7]}"
		# md5=` md5sum /opt/wise_iae/ia12k/process/bin/${file_name} `
		# md5_info_arr=(${md5})
		# file_info="${line}  ${md5_info_arr[0]}"
		# echo "${file_info}" >> ${Check_Result_File}
		# if [ "${iae_tp_time}" != "${file_time}" ]
		# then
			# echo "存在替换进程，可能有未实施的补丁" >> ${Check_Result_File}
			# echo "${line}" >> ${Check_Result_File} 
		# fi
	# done <<< "${all_file_info}"
	# echo >> ${Check_Result_File}
	
	
	# 检查core是否开启
	has_opencore="true"
	iae_pid=` Ssh_Cmd $1 $2 $3 $4 "ssh ${Board_Chip_IP[$1-$5]} cat \"/opt/wise_iae/ia12k/data*/run/iae.pid\" " 2>/dev/null`
	core_size_info=` Ssh_Cmd $1 $2 $3 $4 "ssh ${Board_Chip_IP[$1-$5]} \"cat /proc/${iae_pid}/limits | grep 'core file size'\"" 2>/dev/null `
	core_file_all=`Ssh_Cmd $1 $2 $3 $4 "ssh ${Board_Chip_IP[$1-$5]} \"ls -al /opt/wise_iae/ia12k/data*/corefiles/\"" 2>/dev/null `
	if [ $? -ne 0 ]
	then
		has_opencore="false"
	fi
	
	core_size_info_arr=(${core_size_info})
	soft_limit=${core_size_info_arr[4]}	
	if [[ "${soft_limit}" != "unlimited" ]] || [[ "${has_opencore}" = "false" ]]
	then
		echo "${core_size_info}" >> ${Check_Result_File}
		Output_Warning "$1-$5 maybe not open core  "
	else
		# core开启检查有没有core产生
		core_file_all=` grep ^- <<< "${core_file_all}" `
		if [ $? -eq 0 ]
		then
			echo "$1-$5 program maybe have a exception" >> ${Check_Result_File}
			echo "${core_file_all}" >> ${Check_Result_File}
			Output_Warning "$1-$5 program maybe have a exception"
			echo "${core_file_all}"		# 打印core文件信息	
		else
			echo "$1-$5 program has open core and haven't corefile" >> ${Check_Result_File}
			Output_Info "$1-$5 program has open core and haven't corefile"
		fi
	fi
}
Check_Chip_Gpu_Info(){
	# 无芯片，检查对应的data目录
	date_index=0
	while :
	do
		echo "$1 $2 $3 $4"
		cmd=` Ssh_Cmd $1 $2 $3 $4 "ls /opt/wise_iae/ia12k/data${date_index}" 2>/dev/null `
		if [ $? -ne 0 ]
		then
			break
		fi
		has_opencore="true"
		iae_pid=` Ssh_Cmd $1 $2 $3 $4 "cat /opt/wise_iae/ia12k/data${date_index}/run/iae.pid" 2>/dev/null`
		core_size_info=` Ssh_Cmd $1 $2 $3 $4  "cat /proc/${iae_pid}/limits | grep 'core file size'" 2>/dev/null `
		core_file_all=`Ssh_Cmd $1 $2 $3 $4 "ls -al /root | grep -E \"*core*\" " 2>/dev/null `
		core_size_info_arr=(${core_size_info})
		soft_limit=${core_size_info_arr[4]}	
		ulimit=` Ssh_Cmd $1 $2 $3 $4 "ulimit -c" 2>/dev/null `
		echo ${ulimit}
		if [[ "${soft_limit}"!="unlimited" ]] || [[ "${ulimit}"!="unlimited" ]]
		then
			echo "${core_size_info}" >> ${Check_Result_File}
			Output_Warning "$1-${date_index} maybe not open core  "
		else
			# core开启检查有没有core产生
			if [[ "${core_file_all}" != "" ]]
			then
				echo "$1-${date_index} program maybe have a exception" >> ${Check_Result_File}
				echo "${core_file_all}" >> ${Check_Result_File}
				Output_Warning "$1-$5 program maybe have a exception"
				echo "${core_file_all}"		# 打印core文件信息	
			else
				echo "$1-${date_index} program has open core and haven't corefile" >> ${Check_Result_File}
				Output_Info "$1-${date_index} program has open core and haven't corefile"
			fi
		fi
		((date_index++))
	done
	
	echo "Check_Gpu_Info"
}
#########获取芯片ip映射#######
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码 
##$5 板卡类型，昆仑2：1 OR 昆仑3：2 OR GPU：3
Get_Chip_Ip(){
	Board_Chip_IP_OLD=${!Board_Chip_IP[@]}
    Board_Chip_IP_Key_Arr_OLD=(${Board_Chip_IP_OLD// / })
	for ((ChipI_OLD=0; ChipI_OLD<${#Board_Chip_IP_Key_Arr_OLD[*]}; ChipI_OLD++))
			do
				# echo ${#Board_Chip_IP_Key_Arr_OLD[*]}
				# echo ${ChipI_OLD}
				# echo ${Board_Chip_IP_Key_Arr_OLD[${ChipI_OLD}]}
				unset Board_Chip_IP[${Board_Chip_IP_Key_Arr_OLD[${ChipI_OLD}]}]
			done
	case $5 in
	1)
		# 获取昆仑2板卡芯片数
		PROTOTYPE_NAME=` Ssh_Cmd $1 $2 $3 $4 " cat ${Kunlun2_Type_File} | grep PROTOTYPE_NAME" 2>/dev/null `
		echo ${PROTOTYPE_NAME}
		if [ "$PROTOTYPE_NAME" != "" ];then
			prototypeRet=$(echo $PROTOTYPE_NAME | grep "FB-IA12500-B")
			if [ "$prototypeRet" != "" ];then
				prototypeName="tx1_5"
				#echo $prototypeName
			elif [ -n "$(echo $PROTOTYPE_NAME | grep "FB-IA12500-S")" ];then
				prototypeName="tx1_2"
				#echo $prototypeName
			else
				prototypeName="tx1_10"
				#echo $prototypeName
			fi
		fi
		
		if [ "${prototypeName}" == "tx1_5" ]
		then    # 5芯片昆仑2，第一颗芯片ip：192.168.255.11
			tx1Num=`expr $tx1Num + 5`
			Board_Chip_IP[$1-0]="192.168.255.11"
			Board_Chip_IP[$1-1]="192.168.255.13"
			Board_Chip_IP[$1-2]="192.168.255.15"
			Board_Chip_IP[$1-3]="192.168.255.17"
			Board_Chip_IP[$1-4]="192.168.255.19"
		elif [ "${prototypeName}" == "tx1_2" ]
		then	# 2芯片昆仑2，第一颗芯片ip：192.168.255.17
			tx1Num=`expr $tx1Num + 8`
			Board_Chip_IP[$1-0]="192.168.255.17"
			Board_Chip_IP[$1-1]="192.168.255.19"
		else
			Board_Chip_IP[$1-0]="192.168.255.1"
			Board_Chip_IP[$1-1]="192.168.255.3"
			Board_Chip_IP[$1-2]="192.168.255.5"
			Board_Chip_IP[$1-3]="192.168.255.7"
			Board_Chip_IP[$1-4]="192.168.255.9"
			Board_Chip_IP[$1-5]="192.168.255.11"
			Board_Chip_IP[$1-6]="192.168.255.13"
			Board_Chip_IP[$1-7]="192.168.255.15"
			Board_Chip_IP[$1-8]="192.168.255.17"
			Board_Chip_IP[$1-9]="192.168.255.19"
		fi
		;;
	2)
		SERVICESPATH=/usr/local/sbin/services/iae
		tx1num=` Ssh_Cmd $1 $2 $3 $4 "ls ${SERVICESPATH} | grep 'iae*' | grep -v 'iae_tp'| wc -l" 2>/dev/null `
		tx1num=` expr $tx1num - 1 `
		for tx1num_i in `seq 0 $tx1num`
		do
			hisi_tx1_port="`expr $tx1num_i \* 2 + 1`"
			Board_Chip_IP[$1-${tx1num_i}]="192.168.255.${hisi_tx1_port}"
		done
		;;
	3)
		;;
	*)
		echo	" ERROR "
		My_Exit
	esac
}

##########芯片信息检查##########
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码 
Check_Chip_Info(){
	Chip_Wait_Check=()	# 要检查的芯片号集合
	Chip_Count=0	# 当前板卡芯片数量	
	# 获取板卡类型（昆仑2：1，昆仑3：2，GPU：3）
	engine_version=` sqlite3 /opt/wise/ia12k/db/ias.db "select engine_version from ia_engine group by address having address='\$1'" `
	# 获取芯片ip映射
	Get_Chip_Ip $1 $2 $3 $4 ${engine_version}
	
	# 统计板卡芯片数量
	Board_Chip_IP_Key=${!Board_Chip_IP[@]}
	# echo "Board_Chip_IP=${Board_Chip_IP_Key}"
    Board_Chip_IP_Key_Arr=(${Board_Chip_IP_Key// / })
	Board_Chip_IP_Key_Arr_Check=` grep $1 <<< "${Board_Chip_IP_Key_Arr}" `
  
	# 收集芯片信息
	case ${engine_version} in
	# case 2 in
		1)
			for ((k2ChipI=0; k2ChipI<${#Board_Chip_IP_Key_Arr[*]}; k2ChipI++))
			do
				Check_Kunlun2_Chip_Info $1 $2 $3 $4 "${k2ChipI}"	# 获取芯片信息
			done
			;;
		2)
			for ((k3ChipI=0; k3ChipI<${#Board_Chip_IP_Key_Arr[*]}; k3ChipI++))
			do
				# echo ${#Board_Chip_IP_Key_Arr[*]}
				# echo ${k3ChipI}
				Check_Chip_Hisi_Info $1 $2 $3 $4 "${k3ChipI}" # 获取芯片信息
			done
			;;
		3)
			# Check_Chip_Gpu_Info $1 $2 $3 $4
			;;
		*)
			Output_Error "engine_version of the host ${1} is not supported：${engine_version}"
			My_Exit
	esac
}
###########IA版本检查###########
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码
Check_Board()
{
	echo "Begin Check IAE：$1" >> ${Check_Log}
	echo "IAE Check Result：$1" >> ${Check_Result_File}
	Output_Info "Begin Check  Board $1"
    # 检查IAE软件版本
	Iae_Ver_Info=` Ssh_Cmd $1 $2 $3 $4 "iaecfgtool.sh -v" 2>/dev/null `
	echo "$1 IAE VERSION: " >> ${Check_Log}
	echo "${Iae_Ver_Info}" >> ${Check_Log}	# 记录IAE软件版本
	Iae_Ver_Info_Arr=(${Iae_Ver_Info// / })
	Iae_Hcverm=${Iae_Ver_Info_Arr[3]}  #取出软件版本信息
	Iae_Hcverm_Short=(${Iae_Hcverm#*D})  	# 例HCVERM=IAE60V500R002B02D055SP03，取055SP03做匹配
	Iam_Hcverm_Short=(${Iam_Hcverm#*D})		# IAM版本缩写
	echo "Iae_Version=${Iae_Ver_Info}"
	echo 
	# 记录与IAM版本不一致的板卡信息
	if [ ${Iae_Hcverm_Short} != ${Iam_Hcverm_Short} ]
	then
		echo "$1 the IAE_Version(${Iae_Hcverm}) is not same of IAM_Version(${Iam_Hcverm})" >> ${Check_Result_File}
		Output_Error "$1 the IAE_Version(${Iae_Hcverm}) is not same of IAM_Version(${Iam_Hcverm})"
		#Ver_Warning[$1]=${Iae_Hcverm}
	else
		echo "$1 the IAE_Version(${Iae_Hcverm}) is [ok]" >> ${Check_Result_File}
		Output_Info "$1 the IAE_Version(${Iae_Hcverm}) is [ok]"
	fi
	
	
	# 收集昆仑3信息，engine_version GPU：3；昆仑3：2；昆仑2：1
	engine_version=` sqlite3 /opt/wise/ia12k/db/ias.db "select engine_version from ia_engine group by address having address='\$1'" 2>/dev/null `
	echo "$1 ENGINE VERSION: ${engine_version}" >> ${Check_Log}
	Iae_Drv_Ver=""
	if [ ${engine_version} -eq 2 ]	# 如果是昆仑3板卡
	then
		# 检查昆仑三板卡驱动版本
	    Iae_Drv_Info=`Ssh_Cmd $1 $2 $3 $4 "update.sh -v | grep \"Hisi3559 DRV Version: \"" 2>/dev/null`
		cmd=`grep ${Hisi_Drv_Verson_New} <<< "${Iae_Drv_Info}"`
		if [ $? -ne 0 ]
		then
			Output_Error "the Device_Version is not latest----${Iae_Drv_Info}" 
			echo "$1 the Device_Version is not latest----${Iae_Drv_Info}" >> ${Check_Result_File}
		else
			Output_Info "$1 Device Version is [ok]" 
			echo "$1 Device Version is [ok]" >> ${Check_Result_File}
		fi
	fi
	
	if [ ${engine_version} -eq 2 -o ${engine_version} -eq 3 ]	# 昆路3/GPU检查是否有备份文件
	then
	# 收集昆仑3、GPU进程文件信息
	all_file_info=` Ssh_Cmd $1 $2 $3 $4 "ls -alh /opt/wise_iae/ia12k/process/bin/ | grep iae_ " 2>/dev/null `
	echo "$1 process file info: " >> ${Check_Log}
	echo "${all_file_info}" >> ${Check_Log}		# 记录文件	
	md5=` Ssh_Cmd $1 $2 $3 $4 "md5sum /opt/wise_iae/ia12k/process/bin/iae_*" 2>/dev/null `
	echo "${md5}" >> ${Check_Log}		# 记录MD5值
	iae_tp_info=` Ssh_Cmd $1 $2 $3 $4 "ls -alh /opt/wise_iae/ia12k/process/bin/ | grep iae_tp " 2>/dev/null `
	iae_tp_info_arr=(${iae_tp_info})
	iae_tp_time="${iae_tp_info_arr[5]}-${iae_tp_info_arr[6]}-${iae_tp_info_arr[7]}"
	check_process="true"
	while read line
	do
		file_item_arr=(${line})
		file_name="${file_item_arr[8]}"
		file_time="${file_item_arr[5]}-${file_item_arr[6]}-${file_item_arr[7]}"
		if [[ "${iae_tp_time}" != "${file_time}" ]]		# 文件创建时间与iae_tp不同的都认为是备份文件
		then
			Output_Warning "$1 have a process(/opt/wise_iae/ia12k/process/bin/${file_name}) for backup. \nmaybe have a version or patch not be installed"
			echo "$1 have a process(/opt/wise_iae/ia12k/process/bin/${file_name}) for backup." >> ${Check_Result_File}
			echo "$1 maybe have a version or patch not be installed" >> ${Check_Result_File}
			check_process="false";
		fi
	done <<< "${all_file_info}"
	fi
	if [[ "${check_process}" == "true" ]]
	then
		Output_Info "process file check is [ok]"
		echo "process file check is [ok]" >> ${Check_Result_File}
	fi
	
	# 检查系统资源使用情况（板卡系统资源）
	echo >> ${Check_Log}
	echo "$1 disk use info" >> ${Check_Log}
	all_disk_info=` Ssh_Cmd $1 $2 $3 $4 "df -h" 2>/dev/null `
	echo "${all_disk_info}" >> ${Check_Log}
	Check_Disk "${all_disk_info}"	# 检查磁盘空间，使用超过95%的分区需要给出提示

	echo >> ${Check_Log} 
	echo "$1 memmory use info" >> ${Check_Log} 
	all_memmory_info=` Ssh_Cmd $1 $2 $3 $4 "free" 2>/dev/null `
	echo "${all_memmory_info}" >> ${Check_Log} 
	all_memmory_info=` grep Mem <<< "${all_memmory_info}" `
	Check_Free "${all_memmory_info}"	# 检查内存，使用超过95%的分区需要给出提示
	
	# 检查进程运行状态
	process_running_status=` Ssh_Cmd $1 $2 $3 $4 "iaeserver.sh status | grep Service" 2>/dev/null `
	echo >> ${Check_Log}
	echo "$1 process running status" >> ${Check_Log}
	echo "${process_running_status}" >> ${Check_Log}
	process_all_running="true"
	while read process_status_line
	do
		cmd=` grep Running <<< "${process_status_line}" `
		if [ $? -ne 0 ]
		then
			echo "${process_running_status}" >> ${Check_Result_File}
			Output_Error "have process not running"
			echo "${process_running_status}" 
			process_all_running=false
			break 
		fi
	done <<< "${process_running_status}"
	if [[ "${process_running_status}"!="true" ]]
	then
		echo "check process running status is [ok]" >> ${Check_Result_File}
		Output_Info "check process running status is [ok]"
	fi
	# # CPU信息暂不收集
	# # echo >> ${Check_Result_File} 
	# # Ssh_Cmd $1 $2 $3 $4 "top -H -b" >>${Check_Result_File}
}
###########IA配置检查入口###########
##$1 板卡ip
##$2 板卡ssh端口
##$3 板卡ssh用户名
##$4 板卡ssh密码
Check_IAE()
{	
	address_select_result=` sqlite3 /opt/wise/ia12k/db/ias.db "select address from ia_engine group by address" `
    address_str=(${address_select_result})
	echo "Are the passwords,the users and the ports of all IAE server for SSH the smae?  y/n"
	read login_info_same
	if [[ "${login_info_same}" == "y" ]]
	then
		echo " please input user: "
		read user
		echo " please input password: "
		read password
		echo " please input port: "
		read host_port
		count=${#address_str[*]}
		for((address_i=0; address_i<${#address_str[*]}; address_i++))
		do 
		    # 校验SSH连接
            sshpass -p ${password} ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -p ${host_port} ${user}@${address_str[address_i]} " " 2>/dev/null
	        if [ $? -ne 0 ]
	        then 
				echo "the host $1:${host_port} connection failed" >> ${Check_Log}
				echo "the host $1:${host_port} connection failed" >> ${Check_Result_File}
		        Output_Error "the host ${address_str[address_i]}:${host_port} ${password} ${user} connection failed"
		        continue
	        fi
			Check_Board ${address_str[address_i]} ${host_port} ${user} ${password}		# 在板卡上收集信息
			Check_Chip_Info ${address_str[address_i]} ${host_port} ${user} ${password} 	# 在芯片内部收集芯片信息
			echo "count=${address_i}"
	    done
	else
		for((asi=0; asi<${#address_str[*]}; asi++))
	    do
			echo "please input user for ${address_str[asi]}: "
			read user
			echo "please input password for ${address_str[asi]}: "
			read password
			echo "please input port for ${address_str[asi]}: "
			read host_port
			 # 校验SSH连接
            sshpass -p ${password} ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -p ${host_port} ${user}@${address_str[asi]} " " 2>/dev/null
	        if [ $? -ne 0 ]
	        then 
		        Output_Error "the host ${host_ip}:${host_port} connection failed"
		        continue
	        fi
	    	Check_Board "${address_str[${asi}]}" "${host_port}" "${user}" "${password}"
	    	Check_Chip_Info ${address_str[asi]} ${host_port} ${user} ${password}
		done
	fi
}
Clear_Iam()
{
	Uninstall_Sshpass # 卸载sshpass
	### 恢复ssh授权登录信息
	## 恢复sshd_config
	# 以下配置项删除一个#，恢复配置
	# Sshd_Config_Items=("PubkeyAuthentication" "RSAAuthentication" "AuthorizedKeysFile" "PasswordAuthentication")
	# do
		# Pub_Key_Auth_Rst=` grep -rn "[ ]*${Sshd_Config_Items[${item}]}[ ]*" ${Iam_Sshd_Config} `	# 配置文件中是否有当前配置项
		# while read line
		# do
			# num=${line%%:*}	#所在行
			# config=${line##*:}	# 所在行的所有内容
			# # 如果第一配置项所在行第一个字符是#，就移除
			# first_char=${config:0,1}
			# if ["${first_char}" == "#"]
			# then
				# config=${config#*#}	# 截取第一个#以后字符串
			# fi
			# cmd=` sed -i "${num}c ${config}" ${Iam_Sshd_Config}` 	# 给原配置添加注释
		# done <<< "${Pub_Key_Auth_Rst}"
	# done
	#清空BEGIN ADDED FOR IA CHECK 与 END ADDED FOR IA CHECK之间添加的内容
	# sshd_begin_singal=` grep -rn "BEGIN ADDED FOR IA CHECK" ${Iam_Sshd_Config} `
	# sshd_begin_row=${sshd_begin_singal%%:*}	
	# sshd_end_singal=` grep -rn "END ADDED FOR IA CHECK" ${Iam_Sshd_Config} `	
	# sshd_end_row=${sshd_end_singal%%:*}
	# sed -i "${sshd_begin_row},${sshd_end_row}d" ${Iam_Sshd_Config}	# 删除添加的配置信息
	
	## 恢复ssh秘钥认证信息
	# authKey_begin_singal=` grep -rn "BEGIN ADDED FOR IA CHECK" ${Iam_Authorized_keys} `
	# authKey_begin_row=${authKey_begin_singl%%:*}	
	# authKey_end_singal=` grep -rn "END ADDED FOR IA CHECK" ${Iam_Authorized_keys} `	
	# authKey_end_row=${authKey_end_singal%%:*}
	# sed -i "${authKey_begin_row},${authKey_end_row}d" ${Iam_Authorized_keys}	# 删除添加的配置信息
}
Clear_Iae()
{
	echo "Clear_Iae"
}
######################## 检查硬盘 ###########################
## $1 df -h显示的所有磁盘信息
Check_Disk()
{
	row=0 	# 当前行
	first_line="" 	# 表头
	first_line_printed="false"
	have_error="false"
	while read line
	do
		if [ ${row} -eq 0 ]
		then
			first_line=${line}
			((row++))
			continue
		fi
		disk_info_item="${line}"
		disk_info_item_arr=(${disk_info_item})
		disk_info_item=${disk_info_item_arr[4]}
		cmd=` grep -E '^[0-9]{1,2}%' <<< "${disk_info_item}" `
		if [ $? -ne 0 ]
		then
			disk_info_item=${disk_info_item_arr[3]}
			cmd=` grep -E '^[0-9]{1,2}%' <<< "${disk_info_item}" `
			if [ $? -ne 0 ]
			then
				continue
			fi
		fi
		use_info_num=${disk_info_item%*%}
		((row++))
		if [ ${use_info_num} -gt 95 ]
		then
			have_error="true"
			echo "disk for avaliable is small" >> ${Check_Result_File}
			echo "$1" >> "${Check_Result_File}"
			Output_Warning "disk for avaliable is small"
			echo "$1"	# 打印磁盤信息
			break
		fi
	done <<< "$1"
	if [[ "${have_error}" = "false" ]]
	then
		echo "check disk is [ok]" >> ${Check_Result_File}
		Output_Info "check disk is [ok]"
	fi
}
################ 内存检查 ##############
## $1 free 显示的内存信息
Check_Free()
{
	row=1
	fist_line=""
	fist_line_printed="false"
	have_error="false"
	free_info_item_arr=(${1})
	total=${free_info_item_arr[1]}
	used=${free_info_item_arr[2]}
	use_rate=` echo "scale=2;${used}/${total}"|bc `
	use_rate=` echo "scale=2;${use_rate}*100"|bc `
	use_rate="${use_rate%.*}"
	if [ ${use_rate} -gt 95 ]
	then
		have_error="true"
		echo "memmory for avaliable is small" >> ${Check_Result_File}
		echo "$1" >> "${Check_Result_File}"
		Output_Warning "memmory for avaliable is small, rate: ${use_rate}%"
		echo "$1"	# 打印内存信息
	fi
	if [[ "${have_error}" = "false" ]]
	then
		echo "memmory check is [ok]" >> ${Check_Result_File}
		Output_Info "memmory rate: ${use_rate}%, check is [ok]"
	fi
}
######################## MAIN ###############################
declare -A Ver_Warning  # 存储异常版本信息，KEY：板卡ip；VALUE：异常信息
declare -A Board_Chip_IP  # 存储昆仑2、昆仑3板卡芯片的ip，KEY：板卡ip-芯片号；VALUE：芯片在板卡上的ip
Kunlun2_Type_File='/tmp/.manuinfo'	# 昆仑2板卡类型文件
Kunlun2_Key_File="/root/.ssh/iaCheck"
Iam_Hcverm=''  # iam版本信息
Hisi_Drv_Verson_New='K-R020-D012'  # 昆仑3最新驱动
Check_Result_File="${SHELL_DIR}/CheckResult.txt" # 检查结果输出文件
Check_Log="${SHELL_DIR}/Check.log" # 收集信息输出文件
Iam_Ip="" 	#IAM地址
Iam_Authorized_keys="/root/.ssh/authorized_keys"	# IAM免密登录授权文件
Iam_Sshd_Config="/etc/ssh/sshd_config"	# IAM ssh登录配置文件，需要配置免密登录

if [ $# -eq 0 ] # no para, show usage
then
	Usage
else
	para_in=` getopt f:n:tvehc "$@" ` 
	[ $? -ne 0 ] && exit 1
	eval set -- "${para_in}"
	while true
	do
		case "$1" in
		-c)
			Install_Sshpass
			Check_Iam
			Check_IAE
			break
			;;
		-v)
			echo -e "\nsoft_version : ${SOFT_VERSION}\n"
			break
			;;
		--) break
			;;
		*) 
			exit 1
		esac
			shift
	done
	Clear_Iam
fi
 
